/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.wsif.base;

import java.util.Enumeration;
import java.util.Vector;

import javax.wsdl.WSDLException;
import javax.wsdl.extensions.ExtensibilityElement;
import javax.wsdl.extensions.ExtensionDeserializer;
import javax.wsdl.extensions.ExtensionRegistry;
import javax.wsdl.extensions.ExtensionSerializer;
import javax.wsdl.extensions.UnknownExtensionDeserializer;
import javax.wsdl.extensions.UnknownExtensionSerializer;
import javax.xml.namespace.QName;

import org.apache.wsif.logging.Trc;
import org.apache.wsif.wsdl.extensions.ejb.EJBBindingSerializer;
import org.apache.wsif.wsdl.extensions.format.FormatBindingSerializer;
import org.apache.wsif.wsdl.extensions.java.JavaBindingSerializer;

import com.ibm.wsdl.extensions.PopulatedExtensionRegistry;

/**
 * This is utility class that allows to aggregate multiple
 * extensions registries into one. By default all standard WSDL4J
 * extensions are made available.
 * 
 * @author Alekander Slominski
 * @author Sanjiva Weerawarana
 * @author Owen Burroughs <owenb@apache.org>
 * @author Ant Elder <antelder@apache.org>
 * @author Jeremy Hughes <hughesj@apache.org>
 * @author Mark Whitlock <whitlock@apache.org>
 */

class PrivateCompositeExtensionRegistry extends ExtensionRegistry {
	private static final long serialVersionUID = 1L;
    private Vector extRegs = new Vector();

    PrivateCompositeExtensionRegistry() {
        Trc.entry(this);

        // Add (de)serializers for Java, EJB and Format extensions to the
        // PopulatedExetensionRegistry and add the registry to our list
        PopulatedExtensionRegistry per = new PopulatedExtensionRegistry();
        JavaBindingSerializer javaBindingSerializer = new JavaBindingSerializer();
        javaBindingSerializer.registerSerializer(per);
        FormatBindingSerializer formatSerializer = new FormatBindingSerializer();
        formatSerializer.registerSerializer(per);
        EJBBindingSerializer ejbBindingSerializer = new EJBBindingSerializer();
        ejbBindingSerializer.registerSerializer(per);
        extRegs.add(per);
        Trc.exit();
    }

    public void addExtensionRegistry(ExtensionRegistry reg) {
        Trc.entry(this, reg);
        extRegs.add(reg);
        Trc.exit();
    }

    public void registerSerializer(
        Class parentType,
        Class extensionType,
        ExtensionSerializer es) {
        throw new RuntimeException(
            getClass() + " does not allow to register serializers");
    }

    public void registerDeserializer(
        Class parentType,
        QName elementType,
        ExtensionDeserializer ed) {
        throw new RuntimeException(
            getClass() + " does not allow to register deserializers");
    }

    public ExtensionSerializer querySerializer(
        Class parentType,
        QName extensionType)
        throws WSDLException {
        Trc.entry(this, parentType, extensionType);

        ExtensionSerializer ser;
        Enumeration enum = extRegs.elements();
        while (enum.hasMoreElements()) {
            ExtensionRegistry reg = (ExtensionRegistry) enum.nextElement();
            try {
                ser = reg.querySerializer(parentType, extensionType);
                // Check that we're not looking at the default serializer
                ExtensionSerializer def = reg.getDefaultSerializer();
                if (ser != null && !(ser.equals(def))) {
                    Trc.exit(ser);
                    return ser;
                }
            } catch (WSDLException ex) {
	        	Trc.exception(ex);
                throw ex;
            }
        }
        ser = new UnknownExtensionSerializer();
        Trc.exit();
        return ser;
    }

    public ExtensionDeserializer queryDeserializer(
        Class parentType,
        QName elementType)
        throws WSDLException {
        Trc.entry(this, parentType, elementType);

        ExtensionDeserializer deser;
        Enumeration enum = extRegs.elements();
        while (enum.hasMoreElements()) {
            ExtensionRegistry reg = (ExtensionRegistry) enum.nextElement();
            try {
                deser = reg.queryDeserializer(parentType, elementType);
                // Check that we're not looking at the default deserializer
                ExtensionDeserializer def = reg.getDefaultDeserializer();
                if (deser != null && !(deser.equals(def))) {
                    Trc.exit(deser);
                    return deser;
                }
            } catch (WSDLException ex) {
	        	Trc.exception(ex);
                throw ex;
            }
        }
        deser = new UnknownExtensionDeserializer();
        Trc.exit(deser);
        return deser;
    }

    public ExtensibilityElement createExtension(
        Class parentType,
        QName elementType)
        throws WSDLException {
        Trc.entry(this, parentType, elementType);

        ExtensibilityElement ee;
        Enumeration enum = extRegs.elements();
        while (enum.hasMoreElements()) {
            ExtensionRegistry reg = (ExtensionRegistry) enum.nextElement();
            try {
                ee = reg.createExtension(parentType, elementType);
                Trc.exit(ee);
                return ee;
            } catch (WSDLException ignored) {
	        	Trc.ignoredException(ignored);
            }
        }
        ee = super.createExtension(parentType, elementType);
        Trc.exit(ee);
        return ee;
    }

}
