/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.wsif.wsdl.extensions.java;

import java.io.Serializable;

import javax.wsdl.extensions.ExtensionDeserializer;
import javax.wsdl.extensions.ExtensionRegistry;
import javax.wsdl.extensions.ExtensionSerializer;
import javax.xml.namespace.QName;

import org.apache.wsif.logging.Trc;

import com.ibm.wsdl.Constants;
import com.ibm.wsdl.util.StringUtils;
import com.ibm.wsdl.util.xml.DOMUtils;

/**
 * @author Gerhard Pfau <gpfau@de.ibm.com>
 * @author Owen Burroughs <owenb@apache.org>
 * @author Ant Elder <antelder@apache.org>
 * @author Jeremy Hughes <hughesj@apache.org>
 * @author Mark Whitlock <whitlock@apache.org>
 */
public class JavaBindingSerializer
    implements ExtensionSerializer, ExtensionDeserializer, Serializable {

	private static final long serialVersionUID = 1L;
        
    public void marshall(
        Class parentType,
        QName elementType,
        javax.wsdl.extensions.ExtensibilityElement extension,
        java.io.PrintWriter pw,
        javax.wsdl.Definition def,
        javax.wsdl.extensions.ExtensionRegistry extReg)
        throws javax.wsdl.WSDLException {
        Trc.entry(this, parentType, elementType, extension, pw, def, extReg);

        // CHANGE HERE: Adjust with unmarshall() !!!

        if (extension == null) {
            Trc.exit();
            return;
        }

        if (extension instanceof JavaBinding) {
            JavaBinding javaBinding = (JavaBinding) extension;
            pw.print("      <java:binding");

            Boolean required = extension.getRequired();
            if (required != null) {
                DOMUtils.printQualifiedAttribute(
                    Constants.Q_ATTR_REQUIRED,
                    required.toString(),
                    def,
                    pw);
            }

            pw.println("/>");
        } else if (extension instanceof JavaOperation) {
            JavaOperation javaOperation = (JavaOperation) extension;
            pw.print("      <java:operation");

            if (javaOperation.getMethodName() != null) {
                DOMUtils.printAttribute("methodName", javaOperation.getMethodName(), pw);
            }

            if (javaOperation.getMethodType() != null) {
                DOMUtils.printAttribute("methodType", javaOperation.getMethodType(), pw);
            }

            if (javaOperation.getParameterOrder() != null) {
                DOMUtils.printAttribute(
                    "parameterOrder",
                    StringUtils.getNMTokens(javaOperation.getParameterOrder()),
                    pw);
            }

            if (javaOperation.getReturnPart() != null) {
                DOMUtils.printAttribute("returnPart", javaOperation.getReturnPart(), pw);
            }

            Boolean required = extension.getRequired();
            if (required != null) {
                DOMUtils.printQualifiedAttribute(
                    Constants.Q_ATTR_REQUIRED,
                    required.toString(),
                    def,
                    pw);
            }

            pw.println("/>");
        } else if (extension instanceof JavaAddress) {
            JavaAddress javaAddress = (JavaAddress) extension;
            pw.print("      <java:address");

            if (javaAddress.getClassName() != null) {
                DOMUtils.printAttribute("className", javaAddress.getClassName(), pw);
            }

            if (javaAddress.getClassPath() != null) {
                DOMUtils.printAttribute("classPath", javaAddress.getClassPath(), pw);
            }

            if (javaAddress.getClassLoader() != null) {
                DOMUtils.printAttribute("classLoader", javaAddress.getClassLoader(), pw);
            }

            Boolean required = extension.getRequired();
            if (required != null) {
                DOMUtils.printQualifiedAttribute(
                    Constants.Q_ATTR_REQUIRED,
                    required.toString(),
                    def,
                    pw);
            }

            pw.println("/>");
        }
        Trc.exit();
    }

    /**
     * Registers the serializer.
     */
    public void registerSerializer(ExtensionRegistry registry) {
        Trc.entry(this, registry);
        // binding	
        registry.registerSerializer(
            javax.wsdl.Binding.class,
            JavaBindingConstants.Q_ELEM_JAVA_BINDING,
            this);
        registry.registerDeserializer(
            javax.wsdl.Binding.class,
            JavaBindingConstants.Q_ELEM_JAVA_BINDING,
            this);
        registry.mapExtensionTypes(
            javax.wsdl.Binding.class,
            JavaBindingConstants.Q_ELEM_JAVA_BINDING,
            JavaBinding.class);

        // operation
        registry.registerSerializer(
            javax.wsdl.BindingOperation.class,
            JavaBindingConstants.Q_ELEM_JAVA_OPERATION,
            this);
        registry.registerDeserializer(
            javax.wsdl.BindingOperation.class,
            JavaBindingConstants.Q_ELEM_JAVA_OPERATION,
            this);
        registry.mapExtensionTypes(
            javax.wsdl.BindingOperation.class,
            JavaBindingConstants.Q_ELEM_JAVA_OPERATION,
            JavaOperation.class);
        // address
        registry.registerSerializer(
            javax.wsdl.Port.class,
            JavaBindingConstants.Q_ELEM_JAVA_ADDRESS,
            this);
        registry.registerDeserializer(
            javax.wsdl.Port.class,
            JavaBindingConstants.Q_ELEM_JAVA_ADDRESS,
            this);
        registry.mapExtensionTypes(
            javax.wsdl.Port.class,
            JavaBindingConstants.Q_ELEM_JAVA_ADDRESS,
            JavaAddress.class);
        Trc.exit();
    }

    public javax.wsdl.extensions.ExtensibilityElement unmarshall(
        Class parentType,
        javax.xml.namespace.QName elementType,
        org.w3c.dom.Element el,
        javax.wsdl.Definition def,
        javax.wsdl.extensions.ExtensionRegistry extReg)
        throws javax.wsdl.WSDLException {
        Trc.entry(this, parentType, elementType, el, def, extReg);

        // CHANGE HERE: Use only one temp string ...

        javax.wsdl.extensions.ExtensibilityElement returnValue = null;

        if (JavaBindingConstants.Q_ELEM_JAVA_BINDING.equals(elementType)) {
            JavaBinding javaBinding = new JavaBinding();
            Trc.exit(javaBinding);
            return javaBinding;
        } else if (JavaBindingConstants.Q_ELEM_JAVA_OPERATION.equals(elementType)) {
            JavaOperation javaOperation = new JavaOperation();

            String methodName = DOMUtils.getAttribute(el, "methodName");
            //String requiredStr = DOMUtils.getAttributeNS(el, Constants.NS_URI_WSDL, Constants.ATTR_REQUIRED);
            if (methodName != null) {
                javaOperation.setMethodName(methodName);
            }

            String methodType = DOMUtils.getAttribute(el, "methodType");
            if (methodType != null) {
                javaOperation.setMethodType(methodType);
            }

            String parameterOrder = DOMUtils.getAttribute(el, "parameterOrder");
            if (parameterOrder != null) {
                javaOperation.setParameterOrder(parameterOrder);
            }

            String returnPart = DOMUtils.getAttribute(el, "returnPart");
            if (returnPart != null) {
                javaOperation.setReturnPart(returnPart);
            }
            Trc.exit(javaOperation);
            return javaOperation;
        } else if (JavaBindingConstants.Q_ELEM_JAVA_ADDRESS.equals(elementType)) {
            JavaAddress javaAddress = new JavaAddress();

            String className = DOMUtils.getAttribute(el, "className");
            if (className != null) {
                javaAddress.setClassName(className);
            }

            String classPath = DOMUtils.getAttribute(el, "classPath");
            if (classPath != null) {
                javaAddress.setClassPath(classPath);
            }

            String classLoader = DOMUtils.getAttribute(el, "classLoader");
            if (classLoader != null) {
                javaAddress.setClassLoader(classLoader);
            }
            Trc.exit(javaAddress);
            return javaAddress;
        }
        Trc.exit(returnValue);
        return returnValue;
    }
}