/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package wsdl;

import java.io.File;
import java.io.FileWriter;
import java.net.URL;
import java.util.Properties;

import javax.wsdl.Definition;
import javax.wsdl.WSDLException;
import javax.wsdl.factory.WSDLFactory;
import javax.wsdl.xml.WSDLReader;
import javax.wsdl.xml.WSDLWriter;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;
import util.TestUtilities;

import com.ibm.wsdl.Constants;

/**
 * Junit test to test the deserializing and serializing
 * of a wsdl document including testing reading extension
 * attributes on Parts.
 * @author Owen Burroughs
 * @author Jeremy Hughes <hughesj@apache.org>
 */
public class WSDLTest extends TestCase {
    private final static String DEF_FACTORY_PROPERTY_NAME =
        "javax.wsdl.factory.DefinitionFactory";
    private final static String PRIVATE_DEF_FACTORY_CLASS =
        "org.apache.wsif.wsdl.WSIFWSDLFactoryImpl";

    private static String WSDL_GOOD = "";
    private static String WSDL_BAD = "";
    private static String WSDL_OUT = "";
    private static boolean debugging = true;

    public static void main(String[] args) {
        junit.textui.TestRunner.run(suite());
    }

    public static Test suite() {
        return new TestSuite(WSDLTest.class);
    }

    public WSDLTest(String arg0) {
        super(arg0);
    }

    protected void setUp() {
        WSDL_GOOD = TestUtilities.getWsdlPath("java\\test\\wsdl") + "WSDLTestGood.wsdl";
        WSDL_BAD = TestUtilities.getWsdlPath("java\\test\\wsdl") + "WSDLTestBad.wsdl";
        WSDL_OUT = TestUtilities.getWsdlPath("java\\test\\wsdl") + "WSDLTestOut.wsdl";
    }

    public void testReadWriteGoodWSDL() {
        debug("--- Try good WSDL ---");
        Definition d = readWSDL(null, WSDL_GOOD);
        assertNotNull("Definition is null after reading wsdl", d);
        if (d == null) {
            fail("--- Cannot write wsdl since definition is null ---");
        } else {
            debug("--- Definition read OK ---");
            boolean ok = writeWSDL(d);
            assertTrue("Writing WSDL failed", ok);
        }
    }

    public void testReadBadWSDL() {
        debug("--- Try bad WSDL, expect exception ---");
        Definition d2 = readWSDL(null, WSDL_BAD);
        assertNull("Bad wsdl was read without error!!", d2);
    }

    public void testReadMissingDefaultXMLNamepsaceWSDL() {
        debug("--- Test WSDL with missing default XML namespace is good");
        Definition d =
            readWSDL(null, TestUtilities.getWsdlPath("java\\test\\wsdl") + "WSDLTestNested2.wsdl");
        assertNotNull("Definition is null after reading WSDLTestNested2.wsdl", d);
    }
    
    private static Definition readWSDL(URL contextURL, String wsdlLoc) {
        try {
            Properties props = System.getProperties();
            String oldPropValue = props.getProperty(DEF_FACTORY_PROPERTY_NAME);

            props.setProperty(DEF_FACTORY_PROPERTY_NAME, PRIVATE_DEF_FACTORY_CLASS);

            WSDLFactory factory = WSDLFactory.newInstance();
            WSDLReader wsdlReader = factory.newWSDLReader();
            wsdlReader.setFeature(Constants.FEATURE_VERBOSE, false);
            String context = null;
            if (contextURL != null)
                context = contextURL.toString();
            Definition def = wsdlReader.readWSDL(context, wsdlLoc);

            if (oldPropValue != null) {
                props.setProperty(DEF_FACTORY_PROPERTY_NAME, oldPropValue);
            } else {
                props.remove(DEF_FACTORY_PROPERTY_NAME);
            }
            return def;
        } catch (WSDLException e) {
            debug("EXCEPTION: " + e.getMessage());
            return null;
        }
    }

    private static boolean writeWSDL(Definition def) {
        try {
            WSDLFactory factory = WSDLFactory.newInstance();
            WSDLWriter wsdlWriter = factory.newWSDLWriter();
            File f = new File(WSDL_OUT);
            FileWriter out = new FileWriter(f);
            wsdlWriter.writeWSDL(def, out);
            out.close();
            debug("--- Definition written OK ---");
            return true;
        } catch (Exception e) {
            debug("EXCEPTION: " + e);
            return false;
        }
    }

    private static void debug(String s) {
        if (debugging)
            System.out.println(s);
    }
}
